#include"MS5611.h"
#include <Wire.h>

MS5611 baro; //Horizon

enum BaroState { //Horizon
STATE_IDLE,
STATE_START_PRESSURE,
STATE_WAIT_PRESSURE,
STATE_START_TEMP,
STATE_WAIT_TEMP
};
static BaroState baroState;//Horizon
uint8_t ct; //Horizon
uint32_t rawPressure = 0;//Horizon
uint32_t rawTemperature = 0;//Horizon
int32_t lastPressure = 0;//Horizon Final computed pressure
float baroFiltered = 0;
float groundAltitude = 0;
unsigned long measureStart = 0;
unsigned long prevLoopTime = 0;

void setup() {
  // put your setup code here, to run once:
    Serial.begin(115200);
Wire.begin();
Wire.setClock(400000L); // Communication with MPU-6050 at 400KHz
if (!baro.begin(MS5611_STANDARD)) {
  Serial.println(F("MS5611 not detected."));
  delay(200);
  } else {
    Serial.println(F("MS5611 initialized"));
    ct = baro.getConversionTime();
    Serial.println(ct);
    delay(200);
    }
}

void loop() {
  GetBaroAltitude();
  delay(1);
  float time = millis() - prevLoopTime;
  prevLoopTime = millis();
  Serial.print("main loop:");
  Serial.println(time);
}

float GetBaroAltitude() {
  updateMS5611NonBlocking();
  float rawAltitude = baro.getAltitude(lastPressure);
  if(0.98 * baroFiltered == 0) { //Because first loop alphaBaro*baroFiltered = 0
    baroFiltered = rawAltitude;
  } else {
    baroFiltered = 0.98 * baroFiltered + (1.0f - 0.98) * rawAltitude;
  }
  //Serial.print("/");
  //Serial.print((rawAltitude-groundAltitude) * 100);   
  Serial.print("/rawAltitude/");
  Serial.print(rawAltitude * 100);  
  //Serial.print("/baroFiltered/");  
  //Serial.print(baroFiltered * 100);  
  return baroFiltered;
} //Horizon

void updateMS5611NonBlocking() {
  unsigned long now = millis();
  
  switch (baroState) {
    case STATE_IDLE:
      // Start pressure conversion
      Wire.beginTransmission(MS5611_ADDRESS);
      Wire.write(MS5611_CMD_CONV_D1 + baro.getOversampling());
      Wire.endTransmission();

      measureStart = now;

      //Serial.print(F("/IdleState/"));
      baroState = STATE_WAIT_PRESSURE;
      break;

    case STATE_WAIT_PRESSURE:
      // Has enough time passed?
      if (now - measureStart >= ct) {
        // Read raw pressure (no delay)
        rawPressure = baro.readADC(MS5611_CMD_ADC_READ);

        // Next: start temperature conversion
        Wire.beginTransmission(MS5611_ADDRESS);
        Wire.write(MS5611_CMD_CONV_D2 + baro.getOversampling());
        Wire.endTransmission(); 

        measureStart = now;
        //Serial.print("/waitPressureState/");
        //Serial.print(rawPressure*100);   
        baroState = STATE_WAIT_TEMP;
      }
      break;

    case STATE_WAIT_TEMP:
      // 3) Check if time has passed to read Temperature
      if (now - measureStart >= ct) {
        rawTemperature = baro.readADC(MS5611_CMD_ADC_READ);
        lastPressure = baro.readPressureMAT(true, (int32_t)rawPressure, (int32_t)rawTemperature);
        Serial.print("/waitTempState:press/");
        Serial.print(lastPressure*100);
        baroState = STATE_IDLE;
      }
      break;
  }
}